close all;
clear all;
clc;

xStep = 0.01;

% domain
xDom = -1:xStep:1;

% local and global functions
f1 = exp(xDom);
f2 = exp(-xDom);
fTot = f1 + f2;

% derivatives of the local functions
f1D = diff(f1) / xStep;
f2D = diff(f2) / xStep;
%
f1DD = diff(f1D) / xStep;
f2DD = diff(f2D) / xStep;

% point where to approximate
x1Index = ceil( 3 * numel(xDom) / 4 );
x2Index = floor( numel(xDom) / 4 );

% second order approximations of the local functions
a1 = f1DD(x1Index);
b1 = xDom(x1Index) - f1D(x1Index) / a1;
c1 = f1(x1Index) - 0.5 * a1 * ( xDom(x1Index) - b1 )^2;
q1 = 0.5 * a1 * ( xDom - b1 ).^2 + c1;
%
a2 = f2DD(x2Index);
b2 = xDom(x2Index) - f2D(x2Index) / a2;
c2 = f2(x2Index) - 0.5 * a2 * ( xDom(x2Index) - b2 )^2;
q2 = 0.5 * a2 * ( xDom - b2 ).^2 + c2;

% approximation of the global function
qTot = q1 + q2;

% DEBUG
figure(1)
hold on;
plot(xDom, f1, 'r');
plot(xDom, f2, 'b');
plot(xDom, fTot, 'k');
plot(xDom, q1, '--r');
plot(xDom, q2, '--b');
plot(xDom, qTot, '--k');


strBaseName = 'data__sum_of_local_cost_functions';
strFileNamef1 = sprintf('%s__f1.txt', strBaseName );
strFileNamef2 = sprintf('%s__f2.txt', strBaseName );
strFileNameq1 = sprintf('%s__q1.txt', strBaseName );
strFileNameq2 = sprintf('%s__q2.txt', strBaseName );
strFileNamefTot = sprintf('%s__fTot.txt', strBaseName );
strFileNameqTot = sprintf('%s__qTot.txt', strBaseName );
%
% open the file
fidf1 = fopen(strFileNamef1, 'w');
fidf2 = fopen(strFileNamef2, 'w');
fidq1 = fopen(strFileNameq1, 'w');
fidq2 = fopen(strFileNameq2, 'w');
fidfTot = fopen(strFileNamefTot, 'w');
fidqTot = fopen(strFileNameqTot, 'w');
%
% write the header
fprintf(fidf1, 'x y\n');
fprintf(fidf2, 'x y\n');
fprintf(fidq1, 'x y\n');
fprintf(fidq2, 'x y\n');
fprintf(fidfTot, 'x y\n');
fprintf(fidqTot, 'x y\n');
%
% write the data
for ix = 1:10:numel(xDom);
	%
	fprintf(fidf1,													...
			'%.4f %.4f\n',											...
			xDom(ix), f1(ix)	);
	%
	fprintf(fidf2,													...
			'%.4f %.4f\n',											...
			xDom(ix), f2(ix)	);
	%
	fprintf(fidq1,													...
			'%.4f %.4f\n',											...
			xDom(ix), q1(ix)	);
	%
	fprintf(fidq2,													...
			'%.4f %.4f\n',											...
			xDom(ix), q2(ix)	);
	%
	fprintf(fidfTot,												...
			'%.4f %.4f\n',											...
			xDom(ix), fTot(ix)	);
	%
	fprintf(fidqTot,												...
			'%.4f %.4f\n',											...
			xDom(ix), qTot(ix)	);
	%
end;%
%
% write the last point
fprintf(fidf1,													...
		'%.4f %.4f\n',											...
		xDom(end), f1(end)	);
%
fprintf(fidf2,													...
		'%.4f %.4f\n',											...
		xDom(end), f2(end)	);
%
fprintf(fidq1,													...
		'%.4f %.4f\n',											...
		xDom(end), q1(end)	);
%
fprintf(fidq2,													...
		'%.4f %.4f\n',											...
		xDom(end), q2(end)	);
%
fprintf(fidfTot,												...
		'%.4f %.4f\n',											...
		xDom(end), fTot(end)	);
%
fprintf(fidqTot,												...
		'%.4f %.4f\n',											...
		xDom(end), qTot(end)	);
%
% close the files
fclose(fidf1);
fclose(fidf2);
fclose(fidq1);
fclose(fidq2);
fclose(fidfTot);
fclose(fidqTot);


xDom(x1Index)
f1(x1Index)

xDom(x2Index)
f2(x2Index)

fTot( floor(end/2) )
